#!/bin/bash

###############################################################################################
# config
###############################################################################################

export PATH="/bin:/sbin:/usr/bin:/usr/sbin"

###############################################################################################
# functions
###############################################################################################

# add ifcfg1-12 pre-configured
for((i=1;i<12;i++));
do
cat >/etc/sysconfig/network/ifcfg-eth$i <<EOT
DEVICE="eth$i"
BOOTPROTO="dhcp"
STARTMODE="hotplug"
NM_CONTROLLED="no"
EOT
done

# Don't waste too much time waiting for eth1 -- eth11
#
# The idea here is that network.service only waits for eth0 -- once it's up it should
# return success (the wicked dhcp client of course continues to run on all interfaces).
echo "MANDATORY_DEVICES=\"eth0\"" >> /etc/sysconfig/network/config
# This would allow us to set a long wait time here, as network.service would exit
# early as soon as all mandatory devices are up. This used to work this way on old
# SUSE (2010 era), but does no longer, probably since the switch to wicked.
#
# We set a short WAIT here to avoid losing time.
# This would normally be risky, unless someone else ensures we wait for the network
# before it's needed. On openSUSE-42.x we have added the cloud-init-wait-mds.service
# so we are safe and can tune this wait here to a really low value.
sed -i 's@WAIT_FOR_INTERFACES="[0-9]*"@WAIT_FOR_INTERFACES="1"@' /etc/sysconfig/network/config

# New versions of cloud-init-local bring up the network temporarily using dhclient
# dhclient wastes 5s of time, waiting for a second DHCP server responding ...
sed -i 's@select\-timeout 5;@select-timeout 1;@' -i /etc/dhclient.conf

# Name resolution from dhcp-server
#cat >/etc/resolv.conf <<EOT
#search eu-de.otc-service.com eu-de.otc.t-systems.com otc.t-systems.com openstacklocal
#nameserver 100.125.4.25
#EOT

rm /etc/resolv.conf

# NTP
cat >/etc/ntp.conf <<EOT
# server 127.127.1.0            # local clock (LCL)
# fudge  127.127.1.0 stratum 10 # LCL is unsynchronized
# OTC-Vendordata-NTP-2
server de.pool.ntp.org
server nl.pool.net.org
restrict -4 default notrap nomodify nopeer noquery
restrict -6 default notrap nomodify nopeer noquery
#restrict localnet/netbits nomodify notrap
restrict 127.0.0.1
restrict ::1
driftfile /var/lib/ntp/drift/ntp.drift # path for drift file
logfile   /var/log/ntp          # alternate log file
keys /etc/ntp.keys              # path for keys file
trustedkey 1                    # define trusted keys
requestkey 1                    # key (7) for accessing server variables
controlkey 1                    # key (6) for accessing server variables
EOT

#systemctl enable ntpd

cat > /etc/systemd/timesyncd.conf <<EOT
#  This file is part of systemd.
#
#  systemd is free software; you can redistribute it and/or modify it
#  under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 2.1 of the License, or
#  (at your option) any later version.
#
# Entries in this file have been configured during the OTC image build.
# You can change settings by editing this file.
# Defaults can be restored by simply deleting this file.
#
# See timesyncd.conf(5) for details.
# OTC-Vendordata-NTP-2

[Time]
NTP=de.pool.ntp.org se.pool.ntp.org
FallbackNTP=0.de.pool.ntp.org 1.de.pool.ntp.org 2.de.pool.ntp.org 3.de.pool.ntp.org
EOT

#timedatectl set-ntp true
ln -sf /usr/lib/systemd/system/systemd-timesyncd.service /etc/systemd/system/sysinit.target.wants/

# Set time zone to CET/CEST
# ln -sf /usr/share/zoninfo/Europe/Berlin /etc/localtime

# Repositories
echo "Add repositories..."

#echo "#100.125.4.20 smt01-suse.otc-service.com" >> /etc/hosts
#echo "#100.125.4.28 ntp01.otc-service.com" >>/etc/hosts
#echo "#100.125.4.29 ntp02.otc-service.com" >>/etc/hosts
#echo "#100.125.4.25 dns.otc-service.com" >> /etc/hosts
echo "169.254.169.254 metadata.openstacklocal" >> /etc/hosts


cat >/etc/cloud/cloud.cfg <<EOT
# The top level settings are used as module
# and system configuration.

# A set of users which may be applied and/or used by various modules
# when a 'default' entry is found it will reference the 'default_user'
# from the distro configuration specified below
users:
   - default

# If this is set, 'root' will not be able to ssh in and they
# will get a message to login instead as the above \$user (ubuntu/linux)
disable_root: true

# This will cause the set+update hostname module to not operate (if true)
#preserve_hostname: false

# Keep instance link: Remove /var/lib/cloud/instance manually
# when creating a new image from a VM
#manual_cache_clean: true

# Tweak Ec2 datasource
#datasource:
#   Ec2:
#     metadata_urls: [ 'http://169.254.169.254/latest/' ]
#     timeout: 36 # (defaults to 50 seconds)
#     max_wait: 72 # (defaults to 120 seconds)
datasource_list: [ NoCloud, OpenStack, ConfigDrive, Ec2, OpenNebula, Azure, AltCloud, OVF, MAAS, GCE, CloudSigma, CloudStack, None ]

# Disable networking setup
network:
   config: disabled

# The modules that run in the 'init' stage
cloud_init_modules:
 - migrator
 - seed_random
 - bootcmd
 - write-files
 - growpart
 - resizefs
 - set_hostname
 - update_hostname
 - update_etc_hosts
 - ca-certs
 - rsyslog
 - users-groups
 - ssh

# The modules that run in the 'config' stage
cloud_config_modules:
# Emit the cloud config ready event
# this can be used by upstart jobs for 'start on cloud-config'.
# - emit_upstart
 - disk_setup
 - mounts
 - ssh-import-id
 - locale
 - set-passwords
 - package-update-upgrade-install
 - landscape
 - timezone
 - puppet
 - chef
 - salt-minion
 - mcollective
 - disable-ec2-metadata
 - runcmd
 - byobu

# The modules that run in the 'final' stage
cloud_final_modules:
 - rightscale_userdata
 - scripts-vendor
 - scripts-per-once
 - scripts-per-boot
 - scripts-per-instance
 - scripts-user
 - ssh-authkey-fingerprints
 - keys-to-console
 - phone-home
 - final-message
 - power-state-change

# System and/or distro specific settings
# (not accessible to handlers/transforms)
system_info:
   # This will affect which distro class gets used
   distro: opensuse
   # Default user name + that default users groups (if added/used)
   default_user:
     name: linux
     lock_passwd: False
     gecos: Cloud User
     groups: users
     passwd: \$6\$4H65c8D0zviTG3Dh\$mi0r/W.xpVuvg5v86AsLpqzKN87Y/zMrKP4cKvfdiULxG/PrB96supUAo9wud9ISqTVq3Y3oDFJHLKMe3XVSP.
     sudo: ["ALL=(ALL) NOPASSWD:ALL"]
     shell: /bin/bash
   # Other config here will be given to the distro class and/or path classes
   paths:
      cloud_dir: /var/lib/cloud/
      templates_dir: /etc/cloud/templates/
   ssh_svcname: sshd
ssh_genkeytypes: ['rsa', 'dsa', 'ecdsa' ,'ed25519']

chpasswd:
   list: |
     linux:RANDOM
   expire: False

# vim:syntax=yaml
EOT

# Read random pass from cloud-init log and output & protect
cat >/usr/lib/cloud-init/grabrandompass <<EOT
#!/bin/bash
CIOUT=/var/log/cloud-init-output.log
RPASS=\$(grep -A8 'Set the following .random. passwords' \$CIOUT | grep 'linux:' | tail -n1)
if test -n "\$RPASS"; then
	RPASS=\${RPASS//&/\\\\&}
	sed -i "s~^.*[Ll]ogin with .*\$~Initial login with \$RPASS~" /etc/issue
fi
chmod o-r /etc/issue \$CIOUT
echo -e "Set the following 'random' passwords\n\$RPASS"
EOT
chmod +x /usr/lib/cloud-init/grabrandompass

# By default, multi-user.target (the default), does NOT pull in
# all the cloud-init units which are all wanted by cloud-init.target
# but not by multi-user.target. By hooking grabrandompass into
# multi-user, pulling in (Wants) cloud-final, and ordering it
# after cloud-config but before getty@.service, we create all
# missing links to make multi-user.target do the full cloud initialization.
cat >/usr/lib/systemd/system/grabrandompass.service <<EOT
[Unit]
Description=Extract random password from cloud-init output log
After=cloud-config.service
Before=console-getty.service serial-getty@.service getty@.service
Wants=cloud-final.service

[Service]
Type=oneshot
ExecStart=/usr/lib/cloud-init/grabrandompass
RemainAfterExit=yes
TimeoutSec=10

# Output needs to appear in instance console output
StandardOutput=journal+console

[Install]
WantedBy=cloud-init.target multi-user.target
EOT
systemctl enable grabrandompass

# Ensure that random passwords get written to cloud-init-output.log
sed -i 's/stderr=False, fallback_to_stdout=False/stderr=True, fallback_to_stdout=True/' /usr/lib/python3*/site-packages/cloudinit/config/cc_set_passwords.py

# OTC optimization:
# Despite cloud-init starting after network.target, the network is not
# always really ready yet and even if it is, the meta-data service (MDS) is
# not always responding quickly. Rather than slowing down every boot by having
# extensive waits in the network setup path, let's insert the wait for MDS
# script here to wait for exactly as long as needed (though not forever).
# This is done using three changes that we inject here:
# 1. /usr/lib/cloud-init/wait-mds script waits for the MDS route
# 2. cloud-init-wait-mds.service calls the script
# 3. cloud-init.service runs after cloud-init-wait-mds.service
# TODO: We could better include this in cloud-init RPM.
cat >/usr/lib/cloud-init/wait-mds <<EOT
#!/bin/bash
MDS_WAIT=200
declare -i ctr=0
declare -i wait=0
#ifup eth0
while [ \$ctr -le \$MDS_WAIT ]; do
	if ip route show | grep '\(169.254.169.254\|default\)'; then
		if ! ip route show | grep '169.254.169.254' >/dev/null; then
			RT=\$(ip route show | grep '^default' | sed 's/default/169.254.169.254/')
			echo "ip route add \$RT"
			ip route add \$RT
		fi
		towait=\$((2+\$wait/2))
		if test \$towait -gt 8; then towait=8; fi
		if curl -sS -m\$towait http://169.254.169.254/openstack/latest >/dev/null; then
			echo "Waited for MDS+route \$ctr, MDS \$wait"
			exit 0
		fi
		let wait+=1
	fi
	let ctr+=1
        echo -n "."
        usleep 600000
done
echo "TIMEOUT"
echo "wait-mds timed out, cloud-init wont't work, VM won't be usable" | logger -p daemon.emerg
exit 1
EOT
chmod +x /usr/lib/cloud-init/wait-mds

cat >/usr/lib/systemd/system/cloud-init-wait-mds.service <<EOT
[Unit]
Description=Wait for metadata service availability (cloud-init)
After=local-fs.target network.target cloud-init-local.service
Before=sshd.service sshd-keygen.service cloud-init.service
Requires=network.target
Wants=local-fs.target cloud-init-local.service

[Service]
Type=oneshot
ExecStart=/usr/lib/cloud-init/wait-mds
RemainAfterExit=yes
TimeoutSec=150

# Output needs to appear in instance console output
StandardOutput=journal+console

[Install]
WantedBy=cloud-init.target multi-user.target
EOT
#systemctl enable cloud-init-wait-mds

sed -i 's/After=local-fs.target network.target cloud-init-local.service/After=local-fs.target network.target cloud-init-local.service cloud-init-wait-mds.service/' /usr/lib/systemd/system/cloud-init.service
#sed -i 's/After=local-fs.target dbus.service isdn.service rdma.service SuSEfirewall2_init.service/After=local-fs.target dbus.service isdn.service rdma.service SuSEfirewall2_init.service cloud-init-local.service/' /usr/lib/systemd/system/wickedd.service
#sed -i "/Before=network-pre.target/iBefore=wickedd.service" /usr/lib/systemd/system/cloud-init-local.service

# End of OTC MDS workaround

# Enable /etc/vimrc.local
echo -e "\nif filereadable(\"/etc/vimrc.local\")\n  source /etc/vimrc.local\nendif" >> /etc/vimrc

# Enable random numbers
systemctl enable rng-tools

# Disable DHCLIENT_SET_HOSTNAME
sed -i 's/DHCLIENT_SET_HOSTNAME="yes"/DHCLIENT_SET_HOSTNAME="no"/g' /etc/sysconfig/network/dhcp

# SUSEfirewall2 config
#sed -i 's/FW_SERVICES_EXT_UDP=""/FW_SERVICES_EXT_UDP="68"/' /etc/sysconfig/SuSEfirewall2
sed -i 's/FW_CONFIGURATIONS_EXT=""/FW_CONFIGURATIONS_EXT="sshd"/' /etc/sysconfig/SuSEfirewall2

# SuSEfirewall2-snat depends on enabled SFW2, so continue to use it for now
systemctl enable SuSEfirewall2_init SuSEfirewall2

# Secure settings
cat >/etc/ssh/ssh_config <<EOT
Host *
# ForwardX11Trusted yes
Protocol 2
SendEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
SendEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
SendEnv LC_IDENTIFICATION LC_ALL
StrictHostKeyChecking ask
VerifyHostKeyDNS ask
# ForwardAgent yes
GatewayPorts no
EOT

cat >/etc/ssh/sshd_config <<EOT
Protocol 2
SyslogFacility AUTH
PermitRootLogin forced-commands-only
# Do not enable sshd passwd auth without ensuring really strong passwds (e.g. for deflt user linux)
PasswordAuthentication no
UsePAM yes
X11Forwarding yes
#UseDNS no
Subsystem       sftp    /usr/lib64/ssh/sftp-server
AcceptEnv LANG LC_CTYPE LC_NUMERIC LC_TIME LC_COLLATE LC_MONETARY LC_MESSAGES
AcceptEnv LC_PAPER LC_NAME LC_ADDRESS LC_TELEPHONE LC_MEASUREMENT
AcceptEnv LC_IDENTIFICATION LC_ALL
GSSAPIAuthentication no
GSSAPICleanupCredentials yes
ChallengeResponseAuthentication no
Ciphers aes256-ctr,aes192-ctr,aes128-ctr
KexAlgorithms curve25519-sha256@libssh.org,diffie-hellman-group-exchange-sha256
AllowTcpForwarding no
GatewayPorts no
X11UseLocalhost yes
AllowAgentForwarding yes
PermitTunnel no
LogLevel VERBOSE
RSAAuthentication yes
PubkeyAuthentication yes
PermitEmptyPasswords no
RhostsRSAAuthentication no
HostbasedAuthentication no
IgnoreRhosts yes
PrintMotd no
EOT

cat >/etc/sysctl.d/20-OTC-ipc.conf <<EOT
# OTCvCloud SUSE default parameters
# Can be changed if needed
kernel.sem = 1250 256000 128 8192
kernel.shmall = 1152921504606846720
kernel.shmmax = 9223372036854775807
EOT

cat >/etc/sysctl.d/20-OTC-net.conf << EOT
net.ipv4.ip_local_port_range = 10240 64999

net.core.netdev_max_backlog = 100000
net.core.netdev_budget = 400
# Max memory for options
net.core.optmem_max = 16777216
net.core.rmem_default = 262144
net.core.rmem_max = 16777216
net.core.wmem_default = 262144
net.core.wmem_max = 16777216
# Allow for 128 MB TCP mem always, 640MB max. -- reduce these for small mem configs
#net.ipv4.tcp_mem = 32768 81920 163840
#net.ipv4.udp_mem = 10240 32768 65536
# Bytes to store data in flight (TCP window) per TCP socket (wscale=7)
net.ipv4.tcp_rmem = 4096 87380 6291456
net.ipv4.tcp_wmem = 4096 32768 4194304
# recommended default congestion control is htcp or cubic (default)
#net.ipv4.tcp_congestion_control = htcp
# recommended for hosts with jumbo frames enabled
net.ipv4.tcp_mtu_probing = 1

net.ipv4.conf.all.promote_secondaries = 1
net.ipv4.conf.default.promote_secondaries = 1
net.ipv4.tcp_moderate_rcvbuf = 1
net.ipv4.tcp_no_metrics_save = 1
net.ipv4.tcp_sack = 1
net.ipv4.tcp_window_scaling = 1

# Lower keepalive timer for conn tracking
net.ipv4.tcp_keepalive_time = 500
net.ipv4.tcp_keepalive_intvl = 25
net.ipv4.tcp_keepalive_probes = 15
EOT

cat >/etc/sysctl.d/20-OTC-netopt.conf << EOT
net.core.somaxconn = 8192
net.ipv4.tcp_max_tw_buckets = 16384
#net.ipv4.tcp_tw_recycle = 1
#net.ipv4.tcp_tw_reuse = 1
#net.ipv4.tcp_fin_timeout = 16
#net.ipv4.tcp_synack_retries = 3
EOT

cat >/etc/sysctl.d/20-OTC-vm.conf << EOT
vm.max_map_count = 1000000
vm.swappiness = 20
EOT

cat > /etc/sysctl.d/21-OTC-sec.conf << EOT
# GIS requirements 2015.2
# This should not be changed
kernel.randomize_va_space = 2
kernel.unprivileged_bpf_disabled = 1

net.ipv4.conf.all.accept_redirects = 0
net.ipv4.conf.all.accept_source_route = 0
net.ipv4.conf.all.arp_announce = 1
net.ipv4.conf.all.arp_filter = 1
net.ipv4.conf.all.arp_ignore = 1
net.ipv4.conf.all.rp_filter = 1
net.ipv4.conf.all.secure_redirects=1
net.ipv4.conf.all.send_redirects = 0
net.ipv4.conf.default.accept_redirects = 0
net.ipv4.conf.default.accept_source_route = 0
net.ipv4.conf.default.arp_announce = 1
net.ipv4.conf.default.arp_filter = 1
net.ipv4.conf.default.arp_ignore = 1
net.ipv4.conf.default.rp_filter = 1
net.ipv4.conf.default.send_redirects = 0
net.ipv4.icmp_echo_ignore_broadcasts = 1
net.ipv4.icmp_ignore_bogus_error_responses = 1
net.ipv4.tcp_rfc1337 = 1
net.ipv4.tcp_slow_start_after_idle = 0
net.ipv4.tcp_syncookies = 1
net.ipv6.conf.all.accept_ra = 0
net.ipv6.conf.all.accept_ra_defrtr = 0
net.ipv6.conf.all.accept_ra_pinfo = 0
net.ipv6.conf.all.accept_ra_rt_info_max_plen = 0
net.ipv6.conf.all.accept_ra_rtr_pref = 0
net.ipv6.conf.all.accept_redirects = 0
net.ipv6.conf.all.accept_source_route = 0
net.ipv6.conf.all.autoconf = 0
net.ipv6.conf.all.forwarding = 0
net.ipv6.conf.all.router_solicitations = 0
net.ipv6.conf.default.accept_ra = 0
net.ipv6.conf.default.accept_ra_defrtr = 0
net.ipv6.conf.default.accept_ra_pinfo = 0
net.ipv6.conf.default.accept_ra_rt_info_max_plen = 0
net.ipv6.conf.default.accept_ra_rtr_pref = 0
net.ipv6.conf.default.accept_redirects = 0
net.ipv6.conf.default.accept_source_route = 0
net.ipv6.conf.default.autoconf = 0
net.ipv6.conf.default.forwarding = 0
net.ipv6.conf.default.router_solicitations = 0
#net.ipv4.tcp_timestamps = 0
EOT

# Update repos
zypper refresh
# No autorefresh needed on base repo (does not change)
zypper mr -F Leap_15_4
zypper update --no-recommends --no-confirm

# Login hint
#echo "linux:cloud.1234" | chpasswd
echo -e "Keep in mind that your Keyboard config is set to US layout\n if you have not changed it via user-data\nLogin with linux/cloud.1234" >> /etc/issue

# PW change hint
echo -e '\e[1;31m###################################################################\e[0m' >> /etc/motd
echo -e '\e[1;31m#   Important !!!                                                 #\e[0m' >> /etc/motd
echo -e '\e[1;31m#   Please change password for user linux after first login.      #\e[0m' >> /etc/motd
echo -e '\e[1;31m###################################################################\e[0m' >> /etc/motd
echo -e '\e[1;32mAdapt your keyboard map with sudo loadkeys de/us/... to match yours\e[0m' >> /etc/motd
echo -e '' >> /etc/motd

# Override /etc/resolv.conf with parameter getting from dhcp server
sed -i 's/NETCONFIG_FORCE_REPLACE="no"/NETCONFIG_FORCE_REPLACE="yes"/g' /etc/sysconfig/network/config

# fix openSUSE cloud-init initialize bug, cloud-init version 0.7.6
# also add service cloud-init-local restart to cloud.cfg, but without followin
# testsuite 030_hostname.sh will not work
#cloud-init init
#cloud-init single --name ssh
### disabled, error with cloud-init user_data
#/usr/bin/cloud-init init --local
#/usr/bin/cloud-init init

# Avoid cloud-init decode error on non-UTF8 systemd environment
# This breaks with BOOTABLE_FLAG=\0x80 on kiwi images otherwise.
#patch -d /usr/lib/python3.6/site-packages/cloudinit -p2 < /usr/local/sbin/cloudinit-decodeerr-env1.diff

# Don't wait for language selection
echo "kiwi_oemunattended='true'" >> /.profile

# Fix getty trying to run getty on /dev/getty (SUSE systemd bug)
#systemctl disable getty@getty.service
#systemctl mask getty@getty.service
rm -f /etc/systemd/system/getty.target.wants/getty@.service

exit 0

